"""Module for parsing the WOTC website for data on magic cards"""

import re
from logging import getLogger
from urllib.request import urlopen

import lxml
import discord
from bs4 import BeautifulSoup

import card_py_bot.config
from card_py_bot import WIZARDS_BASE_URL

__log__ = getLogger(__name__)


CARD_ELEMENT_LIST = [
    "Mana Cost",
    "Types",
    "Rarity",
    "Card Text",
    "Flavor Text",
    "P/T",
    "Expansion",
    "Artist",
    "image_url"
]


def parse_cardtextbox(cardtextbox_div) -> str:
    """Parse an arbitrary text box and also extract elements like mana"""
    def extract_content(content):
        content_string_ = ""
        try:
            icon_content = content["alt"]
            icon_string = card_py_bot.config.MANA_DICT[icon_content].strip()
            return " {} ".format(icon_string)
        except (TypeError, KeyError):
            pass

        try:
            for content_prt in content.contents:
                content_string_ += extract_content(content_prt)
        except AttributeError:
            return content
        return content_string_

    content_string = extract_content(cardtextbox_div)

    # cleaning content_string for random newlines
    content_string = content_string.lstrip("\\n").strip()
    return content_string


def parse_base_value(value_div) -> str:
    """Parse out a simple element like an label value name"""
    value_string = value_div.string.strip()
    value_string = re.sub("â€”", "--- ", value_string)
    return value_string


def parse_mana(mana_div) -> str:
    """Parse the mana cost of a card"""
    mana_string = ""
    for mana_content in mana_div.children:
        try:
            mana_string += card_py_bot.config.MANA_DICT[mana_content["alt"]].strip() + " "
        except TypeError:
            pass
    return mana_string


def parse_image(image_div) -> str:
    """Parse the handler url for a card's image"""
    image_handler = image_div.img["src"].lstrip("../../")
    return image_handler


def parse_rarity(rarity_div) -> str:
    """Parse the rarity of a card"""
    rarity = rarity_div.span.string
    return rarity


def parse_artist(artist_div) -> str:
    """Parse the artist name of a card"""
    artist = artist_div.a.string
    return artist


def parse_expansion(expansion_div) -> str:
    """Parse the expansion name and set image handler"""
    second_a = expansion_div.findAll("a")[1].text
    return second_a
    # return WIZARDS_BASE_URL + expansion_div.find("img")["src"].lstrip("../../")


LABEL_DICT = {
    "Community Rating": "skip",
    "Card Name": parse_base_value,
    "Mana Cost": parse_mana,
    "Converted Mana Cost": parse_base_value,
    "Types": parse_base_value,
    "Card Text": parse_cardtextbox,
    "P/T": parse_base_value,
    "Expansion": parse_expansion,
    "Rarity": parse_rarity,
    "All Sets": "skip",
    "Card Number": parse_base_value,
    "Artist": parse_artist,
    "Flavor Text": parse_cardtextbox,
}


def scrape_card(url: str) -> dict:
    """Scrape a WOTC magic card webpage and extract the cards details for
    embedding into a Discord message"""
    __log__.debug("Scraping WOTC Magic card at: {}".format(url))

    soup = BeautifulSoup(urlopen(url), "lxml")

    card_data = dict()
    label_list = soup.find_all("div", class_="label")
    for label_div in label_list:
        for string in label_div.stripped_strings:
            label_name = string.strip(":")
            if label_name in LABEL_DICT and LABEL_DICT[label_name] != "skip":
                card_data[label_name] = \
                    LABEL_DICT[label_name](label_div.find_next_sibling("div"))

    card_image_div = soup.find("div", class_="cardImage")
    card_data["image_url"] = WIZARDS_BASE_URL + parse_image(card_image_div)
    return card_data


def create_card_embed(card_data: dict, card_url: str) -> discord.Embed:
    """Format scraped card data into a Discord embed"""
    try:
        card_name = card_data["Card Name"]
        embed_title = "**Card Name**\n[{}]({})".format(card_name, card_url)
    except Exception:
        embed_title = "Error: giving raw url: {}".format(card_url)

    em = discord.Embed(description=embed_title, colour=0xDEADBF)

    for element in CARD_ELEMENT_LIST:
        if element in card_data:
            if element == "image_url":
                em.set_image(url=card_data[element])
            elif element == "Flavor Text":
                em.add_field(
                    name=element,
                    value="*{}*".format(card_data[element])
                )
            else:
                em.add_field(
                    name=element,
                    value=card_data[element],
                    inline=False
                )
    return em


def embed_card(url: str) -> discord.Embed:
    """From a Magic card url create a Discord embed"""
    return create_card_embed(scrape_card(url), url)



"""card-py-bot Discord bot api"""

from logging import getLogger

from discord.ext import commands

from card_py_bot.config import save_emoji_config, EMOJI_CONFIG_STRING
from card_py_bot.scrape import embed_card

DESCRIPTION = """card-py-bot: An WOTC Magic card link embedding Discord bot!"""

BOT = commands.Bot(command_prefix="?", description=DESCRIPTION)

__log__ = getLogger(__name__)


@BOT.event
async def on_ready():
    """Startup logged callout/setup"""
    __log__.info("logged in as: {}".format(BOT.user.id))


@BOT.event
async def on_message(message):
    """Standard message handler with card and shush functions"""
    if message.content.startswith("http://gatherer.wizards.com/Pages/Card"):
        await BOT.send_message(
            message.channel,
            embed=embed_card(message.content)
        )
    await BOT.process_commands(message)


class Config():
    """Config commands for the card-py-bot"""
    def __init__(self, bot):
        self.bot = bot

    @commands.command()
    @commands.has_permissions(administrator=True)
    async def print_setup(self):
        """Print the emoji config strings for setting up the mana icon config"""
        await self.bot.say(EMOJI_CONFIG_STRING)

    @commands.command(pass_context=True)
    @commands.has_permissions(administrator=True)
    async def save_setup(self, ctx):
        """Save any user printed emoji config strings to the card_py_bot"""
        async for message in self.bot.logs_from(ctx.message.channel, limit=1):
            emoji_ids = [emoji_id.lstrip("\\\\") for emoji_id in message.content.split()[1:]]

            # Save the emoji ids into the emoji_config.json
            save_emoji_config(emoji_ids)


BOT.add_cog(Config(BOT))



"""Emoji config functions"""
import json
import os
import re
from logging import getLogger

from card_py_bot import BASEDIR

__log__ = getLogger(__name__)

# Path where the emoji_config.json will be stored
EMOJI_CONFIG_PATH = os.path.join(BASEDIR, "emoji_config.json")

# Dictionary that is keyed by the Discord short emoji id and the web id
# of each emoji
MANA_ID_DICT = {
    ":15m:": "15",
    ":13m:": "13",
    ":wbm:": "White or Black",
    ":Energy:": "Energy",
    ":10m:": "10",
    ":7m:": "7",
    ":Untap:": "Untap",
    ":brm:": "Black or Red",
    ":bpm:": "Phyrexian Black",
    ":rgm:": "Red or Green",
    ":9m:": "9",
    ":8m:": "8",
    ":1m:": "1",
    ":gum:": "Green or Blue",
    ":2wm:": "Two or White",
    ":wpm:": "Phyrexian White",
    ":4m:": "4",
    ":12m:": "12",
    ":rm:": "Red",
    ":bm:": "Black",
    ":wum:": "White or Blue",
    ":rwm:": "Red or White",
    ":2bm:": "Two or Blue",
    ":gpm:": "Phyrexian Green",
    ":gm:": "Green",
    ":14m:": "14",
    ":bgm:": "Black or Green",
    ":3m:": "3",
    ":5m:": "5",
    ":Tap:": "Tap",
    ":1000000m:": "1000000",
    ":upm:": "Phyrexian Blue",
    ":2gm:": "Two or Green",
    ":rpm:": "Phyrexian Red",
    ":2m:": "2", ":6m:": "6",
    ":2rm:": "Two or Red",
    ":gwm:": "Green or White",
    ":wm:": "White",
    ":um:": "Blue",
    ":16m:": "16",
    ":urm:": "Blue or Red",
    ":ubm:": "Blue or Black",
    ":11m:": "11"
}


def get_emoji_config_string() -> str:
    """Return a string of all the mana ids (in order) for config setup
    in discord"""
    config_string = "?save_setup\n"
    for short_emoji_id in MANA_ID_DICT:
        config_string += ("\\\\{}\n".format(short_emoji_id))

    return config_string


EMOJI_CONFIG_STRING = get_emoji_config_string()


def create_config_json() -> dict:
    """Create and save a blank default config json also return the dict that
    created the json"""
    emoji_config = dict()

    for short_emoji_id in MANA_ID_DICT:
        emoji_config[short_emoji_id] = {
            "web_id": MANA_ID_DICT[short_emoji_id],
            "discord_raw_id": None
        }

    with open(EMOJI_CONFIG_PATH, "w") as file:
        json.dump(emoji_config, file, indent=4)

    return emoji_config


def load_mana_dict() -> dict:
    """Load the emoji config into a mana dict"""
    try:
        with open(EMOJI_CONFIG_PATH, "r") as file:
            emoji_config = json.load(file)
    except FileNotFoundError:
            emoji_config = create_config_json()

    mana_dict = dict()
    for short_emoji_id in emoji_config:
        emoji = emoji_config[short_emoji_id]
        if not emoji["discord_raw_id"]:
            mana_dict[emoji["web_id"]] = "ERROR: NO ID Configured " \
                                         "for {}".format(emoji["web_id"])
        else:
            mana_dict[emoji["web_id"]] = emoji["discord_raw_id"]

    __log__.debug("WOTC Magic mana to Discord emoji "
                  "dictionary constructed: {}".format(mana_dict))
    return mana_dict


MANA_DICT = load_mana_dict()


def parse_raw_emoji_id(raw_emoji_id: str) -> str:
    """Parse a raw emoji id to short emoji id"""
    m = re.search(":[A-Za-z0-9]*:", raw_emoji_id)
    return m.group(0)


def save_emoji_config(raw_emoji_ids):
    """Save the emoji mana config"""
    try:
        with open(EMOJI_CONFIG_PATH, "r") as file:
            emoji_config = json.load(file)
    except FileNotFoundError:
        # if no mana config file is found initialize a new one
        emoji_config = create_config_json()

    for raw_emoji_id in raw_emoji_ids:

        short_emoji_id = parse_raw_emoji_id(raw_emoji_id)

        if short_emoji_id in MANA_ID_DICT:
            emoji_config[short_emoji_id] = {
                    "web_id": MANA_ID_DICT[short_emoji_id],
                    "discord_raw_id": raw_emoji_id
                }
        else:
            raise KeyError("Short Discord emoji id is unknown: "
                           "{}".format(short_emoji_id))

    with open(EMOJI_CONFIG_PATH, "w") as file:
        json.dump(emoji_config, file, indent=4)

    # update MANA_DICT global
    global MANA_DICT
    MANA_DICT = load_mana_dict()



"""card-py-bot

A Discord.py bot server that allows for user typed links of Magic cards (based
off of the http://gatherer.wizards.com/Pages/Card Magic card info repository)
to be automatically embedded into the discord chat. The embedded message
created by the card-py-bot details basic details of the Magic card, as well
as the cards image itself.

Scripts:
    __main__

Modules:
    bot
    config
    scrape

Config:
    emoji_config.json
"""

import os

BASEDIR = os.path.dirname(os.path.realpath(__file__))

WIZARDS_BASE_URL = "http://gatherer.wizards.com/"



"""Main script entry point for the card-py-bot"""
import argparse
import sys

import logging
import logging.handlers

from card_py_bot.bot import BOT
import card_py_bot.config


def main():
    """Startup script for the card-py-bot"""
    parser = argparse.ArgumentParser(description="card-py-bot: a Discord bot "
                                                 "that web scrapes Magic card "
                                                 "links and embeds the "
                                                 "card's details into a "
                                                 "Discord message")

    parser.add_argument("-c", "--emoji-config-path", type=str,
                        default=card_py_bot.config.EMOJI_CONFIG_PATH,
                        dest="emoji_config_path",
                        help="Path to the card-py-bot emoji_config.json to be "
                             "loaded/generated from (default: %(default)s)")

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("-t", "--token", type=str,
                       help="String of the Discord token for the bot")
    group.add_argument("-tf", "--token-file", type=str, dest="token_file",
                       help="Path to file containing the Discord token for "
                            "the bot")

    group = parser.add_argument_group(title="Logging config")
    group.add_argument("-v", "--verbose", action="store_true",
                       help="Enable verbose logging")
    group.add_argument("-f", "--log-dir", dest="logdir",
                       help="Enable time rotating file logging at "
                            "the path specified")
    group.add_argument("-d", "--debug", action="store_true",
                       help="Enable DEBUG logging level")

    args = parser.parse_args()

    # initialize logging
    handlers = list()
    if args.logdir is not None:
        handlers.append(
            logging.handlers.TimedRotatingFileHandler(
                args.logdir,
                when="D",
                interval=1,
                backupCount=45
            )
        )

    if args.verbose:
        handlers.append(logging.StreamHandler())
    if args.debug:
        level = logging.DEBUG
    else:
        level = logging.INFO

    logging.basicConfig(
        level=level,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        handlers=handlers,
    )

    # set the emoji config path
    card_py_bot.config.EMOJI_CONFIG_PATH = args.emoji_config_path

    # read the token file and extract the token
    if args.token_file is not None:
        token_file = open(args.token_file, "r")
        token = str(token_file.read()).strip()
        token_file.close()
    elif args.token is not None:
        token = args.token

    # run the bot
    BOT.run(token)

    return 0


if __name__ == "__main__":
    sys.exit(main())



"""card-py-bot setup"""

from setuptools import setup, find_packages


def readme():
    with open("README.md") as f:
        return f.read()


setup(
    name="card-py-bot",
    version="3.1.2",
    description="A Discord Bot for embedding WOTC Magic card links "
                "into Discord",
    long_description=readme(),
    author="Nathan Klapstein",
    author_email="nklapste@ualberta.ca",
    url="https://github.com/nklapste/card-py-bot",
    download_url="https://github.com/nklapste/card-py-bot/archive/3.1.2.tar.gz",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    packages=find_packages(exclude=["test"]),
    include_package_data=True,
    package_data={
        "": ["README.md"],
        "card_py_bot": ["MANA_ICONS/*.gif"],
    },
    install_requires=[
        "beautifulsoup4",
        "lxml",
        "discord.py",
    ],
    tests_require=["pytest"],
    entry_points={
        "console_scripts": ["card-py-bot = card_py_bot.__main__:main"],
    },
)
